%Satellite initialisation
%
%
%Author: Karianne Knutsen Tnne
%
%Date: 28/9-06
%
global Jx Jy Jz J A I omega_o I_w K_p K_d omega_o_io w0 w_o_io Sd Sp; 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                  %
% Initialization of the satellite dynamic and kinematic modes      %
%                                                                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%%%%%%%%%%%%%%%%Inertial matrix%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Jx = 22.0;
Jy = 32.3;
Jz = 34.7;

J = [Jx 0 0 ; 0 Jy 0; 0 0 Jz];
I = J;

%%%%%%%%%%%%%%%Lunar information%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Orbit =  384400; %km from Earth
diameter =  3472e3; %Polar diameter  Equatorial diameter = 3476e3 
altitude = 100e3; % altitude of orbit
radius = diameter/2; %radius of the moon in meters
mass = 7.35e22; %kg moon mass
G = 6.6742e-11; %Nm^2/kg^2 = m^3/(kg*s^2)   %Universal gravity constant
%%%%%%%%%%%%%Orbit information%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
omega_o= sqrt(G*mass/radius^3); % Orbit Velocity
omega_o_io = [0 omega_o 0]';         % orbit ang.velocity relativ to inertial 

w0 = omega_o;
w_o_io = omega_o_io;


% frame expressed in orbital frame

q_0=euler2q(pi/180*[5 5 5]'); 
w_b_ob=[0.0005 0.0005 0.0005]'; 

R_b_o = rquat(q_0)';
c2_0=R_b_o(:,2);
w_b_ib_0 = w_b_ob + omega_o*c2_0;

selectC = 2; 

% %***************************************************************** 
% Reaction Wheels data
%*****************************************************************
t1 = [sqrt(1/3) sqrt(2/3) 0]';
t2 = [sqrt(1/3) -sqrt(2/3) 0]';
t3 = [-sqrt(1/3) 0 -sqrt(2/3)]';
t4 = [-sqrt(1/3) 0 sqrt(2/3)]';
A = [t1 t2 t3 t4];
I_w = 1e-2*eye(4);
w_w_0 = [0 0 0 0]';

%***************************************************************** 
% Noise data
%*****************************************************************
qNoise = 1e-12; 
wNoise = 1e-8;
  
%***************************************************************** 
% Controller parameters
%*****************************************************************

%%%%%%%%%%%%%%%% PD independent%%%%%%%%%%%%
K_p = 60*eye(3);
K_d = 100*eye(3);

%%%%%%%%%%%%%%%Robust%%%%%%%%%%%%%%%%%%%%%
Gp = 20*eye(3);
Gr = 80*eye(3);
gam = 48;
%%%%%%%%%%%%%%%%PD Dep%%%%%%%%%%%%%%%%%%%%
alpha = 6;
g_d = 0.8; %max(norm(w_b_ob_dot)) tested in simulations to avoid on-line gain scheduling
g_I = norm(I);

Sp = 45;
Sd = 2*g_d*g_I*alpha;

