function F = lin(in) 
%Linearizarion
%
%
%Author: Karianne Knutsen Tnne
%
%Date: 30/10-06
%
global Jx Jy Jz  w0 A 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                 %
%   Linearization of the state matrix for use in the EKF          %
%                                                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
x_hat = in(1:7);
w_w = in(8:11);

kx = ( Jy-Jz )/Jx;
ky = ( Jx-Jz )/Jy;
kz = ( Jy-Jx )/Jz;

eps = x_hat(2:4);          n = x_hat(1);          w_b_ob = x_hat(5:7);
eps1 = eps(1);          eps2 = eps(2);              eps3 = eps(3);
w1 = w_b_ob(1);           w2 = w_b_ob(2);             w3 = w_b_ob(3);

R_b_o = Rquat([n; eps])';
%%%%%%%%%%%%%%%%Necessary factors from the R_b_o matrix%%%%%%%%%%%%%%%%%%% 

c12 =R_b_o(1,2);    %row 1, col 2
c13 =R_b_o(1,3);    %row 1, col 3


c22 =R_b_o(2,2);    %row 2, col 2
c23 =R_b_o(2,3);    %row 2, col 3


c32 =R_b_o(3,2);    %row 3, col 2
c33 =R_b_o(3,3);    %row 3, col 3



%%%%%%%%%%%%%%%%%%%%%%%%ACTUATOR INERTIAL DERIVATION%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%e factor used in the component form of f_actuatorInertial%%%%%
e = zeros(3,1);
for j = 1:3
    for i=1:4
        e(j) = e(j)+1e-3*(A(j,i)*w_w(i) + A(j,i)*(A(1,i)*w1...
            + A(2,i)*w2 + A(3,i)*w3));
    end
end

%%%%%%%%%%%%%%%%%%%%% e derivated with respect to eta - n%%%%%%%%%%%%%%%

de_dn = zeros(3,1);
for j = 1:3
    for i=1:4
        de_dn(j) = de_dn(j) + 1e-3*(A(j,i)*(w_w(i)-A(1,i)*2*eps3*w0)...
            -A(2,i)*2*n*w0+A(3,i)*2*eps1*w0);
    end
end
%%%%%%%%%%%%%%%%%%%%% e derivated with respect to epsilon%%%%%%%%%%%%%%%
%%%%epsilon_1%%%%
de_deps1 = zeros(3,1);
for j = 1:3
    for i=1:4
        de_deps1(j) = de_deps1(j) + 1e-3*(A(j,i)*(w_w(i)-A(1,i)*2*eps2*w0)+...
            A(2,i)*2*eps1*w0+A(3,i)*2*n*w0);
    end
end
%%%%epsilon_2%%%%
de_deps2 = zeros(3,1);
for j = 1:3
    for i=1:4
        de_deps2(j) = de_deps2(j) + 1e-3*(A(j,i)*(w_w(i)-A(1,i)*2*eps1*w0)-...
            A(2,i)*2*eps2*w0+A(3,i)*2*eps3*w0);
    end
end
%%%%epsilon_3%%%%
de_deps3 = zeros(3,1);
for j = 1:3
    for i=1:4
        de_deps3(j) = de_deps3(j) + 1e-3*(A(j,i)*(w_w(i)-A(1,i)*2*n*w0)+...
            A(2,i)*2*eps3*w0+A(3,i)*2*eps2*w0);
    end
end
%%%%%%%%%%%%%%%%%%%%% e derivated with respect to omega%%%%%%%%%%%%%%%
%%%%%w1%%%%%%%%
de_dw1 = zeros(3,1);
for j = 1:3
    for i=1:4
        de_dw1(j) = de_dw1(j) + 1e-3*(A(j,i)*(w_w(i)-A(1,i)));
    end
end
%%%%%w2%%%%%%%%
de_dw2 = zeros(3,1);
for j = 1:3
    for i=1:4
        de_dw2(j) = de_dw2(j) + 1e-3*(A(j,i)*(w_w(i)-A(2,i)));
    end
end

%%%%%w3%%%%%%%%
de_dw3 = zeros(3,1);
for j = 1:3
    for i=1:4
        de_dw3(j) = de_dw3(j) + 1e-3*(A(j,i)*(w_w(i)-A(3,i)));
    end
end
%%%%%%%%%%%%%dF_inertialActuator%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%epsilon1%%%

dFact1_deps1 = (1/Jx)*(de_deps1(2)*(c32*w0 - w3) - 2*e(2)*n*w0 + ...
    de_deps1(3)*(w2 - c22*w0) + 2*e(3)*eps1*w0);

dFact2_deps1 = (1/Jy)*(de_deps1(1)*(-c32*w0 + w3) + 2*e(1)*n*w0 + ...
    de_deps1(3)*(-w1 + c12*w0) - 2*e(3)*eps2*w0);

dFact3_deps1 = (1/Jz)*(de_deps1(1)*(c22*w0 - w2) - 2*e(1)*eps1*w0 + ...
    de_deps1(1)*(w1 - c12*w0) - 2*e(2)*eps2*w0);

%%%epsilon2%%%

dFact1_deps2 = (1/Jx)*(de_deps2(2)*(c32*w0 - w3) + 2*e(2)*eps3*w0 + ...
    de_deps2(3)*(w2 - c22*w0) - 2*e(3)*eps2*w0);

dFact2_deps2 = (1/Jy)*(de_deps2(1)*(-c32*w0 + w3) - 2*e(1)*eps3*w0 + ...
    de_deps2(3)*(-w1 + c12*w0) + 2*e(3)*eps1*w0);

dFact3_deps2 = (1/Jz)*(de_deps2(1)*(c22*w0 - w2) + 2*e(1)*eps2*w0 + ...
    de_deps2(2)*(w1 - c12*w0) - 2*e(2)*eps1*w0);

%%%epsilon2%%%

dFact1_deps3 = (1/Jx)*(de_deps3(2)*(c32*w0 - w3) + 2*e(2)*eps2*w0 + ...
    de_deps3(3)*(w2 - c22*w0) + 2*e(3)*eps3*w0);

dFact2_deps3 = (1/Jy)*(de_deps3(1)*(-c32*w0 + w3) - 2*e(1)*eps2*w0 + ...
    de_deps3(3)*(-w1 + c12*w0) - 2*e(3)*n*w0);

dFact3_deps3 = (1/Jz)*(de_deps3(1)*(c22*w0 - w2) - 2*e(1)*eps3*w0 + ...
    de_deps3(2)*(w1 - c12*w0) - 2*e(2)*n*w0);

%%%%%%omega1%%%%%%
dFact1_dw1 = (1/Jx)*(de_dw1(2)*(c32*w0 - w3) + de_dw1(3)*(w2 - c22*w0)); 

dFact2_dw1 = (1/Jy)*(de_dw1(1)*(-c32*w0 + w3) + de_dw1(3)*(-w1 + c12*w0) - e(3));

dFact3_dw1 = (1/Jz)*(de_dw1(1)*(c22*w0 - w2) + de_dw1(2)*(w1 - c12*w0) + e(2));

%%%%%%omega2%%%%%%
dFact1_dw2 = (1/Jx)*(de_dw2(2)*(c32*w0 - w3) + de_dw2(3)*(w2 - c22*w0) + e(3)); 

dFact2_dw2 = (1/Jy)*(de_dw2(1)*(-c32*w0 + w3) + de_dw2(3)*(-w1 + c12*w0));

dFact3_dw2 = (1/Jz)*(de_dw2(1)*(c22*w0 - w2) + de_dw2(2)*(w1 - c12*w0) - e(1));

%%%%%%omega3%%%%%%
dFact1_dw3 = (1/Jx)*(de_dw3(2)*(c32*w0 - w3) + de_dw3(3)*(w2 - c22*w0) - e(2)); 

dFact2_dw3 = (1/Jy)*(de_dw3(1)*(-c32*w0 + w3) + de_dw3(3)*(-w1 + c12*w0) + e(1));

dFact3_dw3 = (1/Jz)*(de_dw3(1)*(c22*w0 - w2) + de_dw3(2)*(w1 - c12*w0));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

dFact_total =  [dFact1_deps1 dFact1_deps2 dFact1_deps3 dFact1_dw1 dFact1_dw2 dFact1_dw3;
                dFact2_deps1 dFact2_deps2 dFact2_deps3 dFact2_dw1 dFact2_dw2 dFact2_dw3;
                dFact3_deps1 dFact3_deps2 dFact3_deps3 dFact3_dw1 dFact3_dw2 dFact3_dw3];


dFinert_total = [-2*kx*w0*(eps3*w2+eps2*w3) 2*kx*w0*(eps3*w3-eps2*w2) 2*kx*w0*(eps1*w2-n*w3) 0 kx*w3 kx*w2;
                2*ky*w0*(eps1*w3+eps3*w1) 2*ky*w0*(n*w3+eps2*w1) 2*ky*w0*(eps3*w3-eps1*w1) -ky*w3 0 -ky*w1;
                2*kz*w0*(eps1*w2+eps2*w1) 2*kz*w0*(n*w2-eps3*w1) 2*kz*w0*(n*w1+eps3*w2) -kz*w2 -kz*w1 0];
   
dFrot = [2*w0*(eps2*w3 - eps3*w2) -2*w0*(eps3*w3 + eps2*w2) 2*w0*(n*w3 + eps1*w2) 0 -c32*w0 c22*w0;
        2*w0*(eps3*w1 - eps1*w3) 2*w0*(eps2*w1 - n*w3) -2*w0*(eps1*w1 + eps3*w3) c32*w0 0 -c12*w0;
        2*w0*(eps1*w2 - eps2*w1) 2*w0*(eps3*w1 + n*w2) 2*w0*(eps3*w2 - n*w1) -c22*w0 c12*w0 0];

dFgrav = 6*w0^(2)*[kx*(n*c33 - eps1*c23) kx*(eps3*c33 - eps2*c23) kx*(eps2*c33 + eps3*c23) 0 0 0; 
                   ky*(eps3*c33 - eps1*c13) ky*(-n*c33 - eps2*c13) ky*(eps1*c33 + eps3*c13) 0 0 0;
                   kz*(eps3*c23 + n*c13) kz*(eps3*c13 - n*c23) kz*(eps1*c23 + eps2*c13) 0 0 0];


%%%%%%%%%%%%%%%%%%%%%Total linearisation%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
dFvel_dx = dFinert_total + dFact_total + dFgrav + dFrot;                  %
%                                                                         %


dFatt_dx = 0.5*[0 w3 -w2 n -eps3 eps2;
            -w3 0 w1 eps3 n -eps1;
            w2 -w1 0 -eps2 eps1 n];
    
%                                                                         %
F = [dFatt_dx 
    dFvel_dx;];                                                            %
                                                              %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%









