function x_hat = starEKF(in)
%starEKF 
%
%Author: Karianne Knutsen Tnne
%
%Date: 23/10-06
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                 %
%  EKF with use of one star tracker                               %
%                                                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global Pk_next wk_next xk_next Q R H GG qk_next deltaT 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           Input from the Satellite Dynamics                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
q_star= in(1:4);    %measurement quaternions
T_w = in(5:7);   %angular rate of reaction wheels
w_w = in(8:11);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           Extended Kalman Filter                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

xk = xk_next;        
qk = qk_next;

nk = xk(1);
ek = xk(2:4);
wk = xk(5:7);                 
Pk = Pk_next;             


%%%%%%%%%%%%Measurement noise covariance matrix is set%%%%%%%%%%%%%
sigma_v = nk*R + SK_matrix(ek)*R;           %3x1 matrix

Rk_star = diag([abs(sigma_v(1)) abs(sigma_v(2)) abs(sigma_v(3))]);%3x3 matrix

%Rnorm = norm(Rk_star);

%%%%RkMax
%Svec = max(sigma_v);
%Smax = max(Svec);

%%%%%RkMin
%RvecMin = min(Rk_star);
%Rmin = min(RvecMin);


%%%%%%%%%%%%Kalman Gain%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Kk = Pk*H'*inv(H*Pk*H' + Rk_star); %Calculates the Kalman gain 6x3matrix
K_epsilon = Kk(1:3,1:3);           %splits the gain into epsilon and omega
K_omega = Kk(4:6,1:3);

%%%%%%%%%%%Estimation error%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
d_qk = qInvProduct(q_star, qk);        %q_star X qk^(-1) = q_error  Kun ndvendig nr quaternioner brukes, normalt 
d_ek = d_qk(2:4);               %epsilon part of delta q.              bare  trekke forrige estimat fra mlingen
d_ek_w = q_star(2:4)-qk(2:4);   %stjerne pos. - tidliger pos. 

%%%%%%%%%%%%Estimate update%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            %Quaternion estimate%
            
dek = K_epsilon*d_ek;                     %Estimated delta epsilon
nk_est = sqrt(1 - norm(dek,2)^2 * sign(d_qk(1))); %Estimated eta

qk_est = qProduct([nk_est ; dek],qk);  %Estimated q
            
             %Omega estimate%
        
wk_est = wk + K_omega*d_ek_w;          %Estimated omega
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
xk_est = [qk_est; wk_est];               %Total estimate           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%Error covariance update%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
I = eye(6);
P = (I - Kk*H)*Pk*(I - Kk*H)' + Kk*Rk_star*Kk';

%%%%%max value
% Pvec = max(P);
% Pmax = max(Pvec);
% %%%%%%%min value
% PvecMin = min(P);
% Pmin = min(PvecMin);
% 

% Pnorm = norm(P);
%%%%%%%%%%%%%%Forward Euler integration%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
inVerdi = [xk_est; w_w];
F = lin(inVerdi);  %uses the linearized system matrix
% 
% Fdet = det(F);
% Fnorm = norm(F);


Phi_k = eye(6) + F*deltaT +1/2*F^2*deltaT^2;


%%%%%%%%%%%%%%%State estimate propagation%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Phi_tot_1 = nonLin(xk_est, w_w, T_w);
xk_temp = Phi_tot_1*deltaT + xk_est;
Phi_tot_2 = nonLin(xk_temp,w_w,T_w );

xk_next = xk_est + deltaT/2*(Phi_tot_1+Phi_tot_2);
qk_next = xk_next(1:4);
wk_next = xk_next(5:7);

%%%%%%%%%%%%%%%%Normalisation%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%qk_next = qk_next/norm(qk_next,2);
xk_next(1:4) = qk_next;

%%%%%%%%%%%%%%%Error Covariance matrix%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Pk_next = Phi_k*P*Phi_k' + GG*Q*GG';



% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %           Return/Output                                        %
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


x_hat= xk_est;






 


